"""
By accessing or using this script, sample code, application programming interface, tools, and/or associated
documentation (if any) (collectively, “Tools”), You (i) represent and warrant that You are entering into this Agreement
on behalf of a company, organization or another legal entity (“Entity”) that is currently a customer or partner of
CrowdStrike, Inc.(“CrowdStrike”), and (ii) have the authority to bind such Entity and such Entity agrees to be bound by
this Agreement. CrowdStrike grants Entity a non-exclusive, non-transferable, non-sublicensable, royalty free and limited
license to access and use the Tools solely for Entity’s internal business purposes and in accordance with its
obligations under any agreement(s) it may have with CrowdStrike. Entity acknowledges and agrees that CrowdStrike and its
licensors retain all right, title and interest in and to the Tools, and all intellectual property rights embodied
therein, and that Entity has no right, title or interest therein except for the express licenses granted hereunder and
that Entity will treat such Tools as CrowdStrike’s confidential information.

THE TOOLS ARE PROVIDED “AS-IS” WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESS, IMPLIED OR STATUTORY OR OTHERWISE.
CROWDSTRIKE SPECIFICALLY DISCLAIMS ALL SUPPORT OBLIGATIONS AND ALL WARRANTIES, INCLUDING WITHOUT LIMITATION, ALL IMPLIED
WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE, TITLE, AND NON-INFRINGEMENT. IN NO EVENT SHALL
CROWDSTRIKE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
THE USE OF THE TOOLS, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

© Copyright CrowdStrike 2019-20
"""

import argparse
import datetime
import http
import logging
import sys
import requests
import settings
from pymisp import ExpandedPyMISP, MISPObject, MISPEvent, MISPAttribute, MISPOrganisation
import os
import urllib3


class IntelAPIClient:
    """Python Client for Intel API Indicators and Reports

    :param client_id: id of the Crowdstrike client
    :param client_secret: secret key of the Crowdstrike client
    :param crowdstrike_url: Crowdstrike API URL
    """
    def __init__(self, client_id, client_secret, crowdstrike_url):
        self.client_id = client_id
        self.client_secret = client_secret

        self.crowdstrike_url = crowdstrike_url

        self.auth_path = "{}/oauth2/token".format(self.crowdstrike_url)
        self.reports_path = "{}/intel/combined/reports/v1/".format(self.crowdstrike_url)
        self.indicators_path = "{}/intel/combined/indicators/v1/".format(self.crowdstrike_url)
        self.actors_path = "{}/intel/combined/actors/v1/".format(self.crowdstrike_url)

        self.valid_report_types = ["csa", "csir", "csit", "csgt", "csia", "csmr", "csta", "cswr"]
        self.request_size_limit = 1000
        self.request_offset_limit = 100000

        self.__get_auth_token()

    def __get_auth_token(self):
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        data = {"client_id": self.client_id,
                "client_secret": self.client_secret}

        resp = requests.post(self.auth_path, data=data, headers=headers, verify=False, allow_redirects=True)
        if resp.status_code != http.HTTPStatus.CREATED:
            raise Exception("Unable to generate authentification token for Crowdstrike Intel API. "
                            "Status code was {}. Response is {}.".format(resp.status_code, resp))

        resp_json = resp.json()

        access_token = resp_json.get('access_token')
        if not access_token:
            logging.warning('Could not decode access token from json response. Response is {}.'.format(resp_json))
            return

        self.auth_token = access_token

    def get_reports(self, start_time):
        """Get all the reports that were updated after a certain moment in time (UNIX)

        :param start_time: unix time of the oldest report you want to pull
        """
        reports = []
        offset = 0
        total = 0
        first_run = True

        while offset < total or first_run:
            headers = {
                'Authorization': 'Bearer ' + self.auth_token,
                'Content-Type': 'application/json',
            }
            params = {"sort": "last_modified_date.asc",
                      "filter": 'last_modified_date:>{}'.format(start_time),
                      'limit': self.request_size_limit,
                      'offset': offset}

            resp_json = self.__send_request_check_status(self.reports_path, headers, params)

            total = resp_json.get('meta', {}).get('pagination', {}).get('total')
            offset += resp_json.get('meta', {}).get('pagination', {}).get('limit')
            first_run = False

            reports.extend(resp_json.get('resources', []))

        valid_reports = [report for report in reports if self.__is_valid_report(report)]
        return valid_reports

    def __is_valid_report(self, report):
        return any(report.get('name') and report.get('name').lower().startswith(valid_type)
                   for valid_type in self.valid_report_types)

    def get_indicators(self, start_time, include_deleted):
        """Get all the indicators that were updated after a certain moment in time (UNIX)

        :param start_time: unix time of the oldest indicator you want to pull
        :param include_deleted: include indicators marked as deleted
        """
        indicators = []
        indicators_in_request = []
        first_run = True

        while len(indicators_in_request) == self.request_size_limit or first_run:
            headers = {
                'Authorization': 'Bearer ' + self.auth_token,
            }
            params = {"sort": "_marker.asc",
                      "filter": "_marker:>='{}'".format(start_time),
                      'limit': self.request_size_limit,
                      }
            if include_deleted:
                params['include_deleted'] = True

            resp_json = self.__send_request_check_status(self.indicators_path, headers, params)

            first_run = False

            indicators_in_request = resp_json.get('resources', [])
            if not indicators_in_request:
                break

            indicators.extend(indicators_in_request)

            last_marker = indicators_in_request[-1].get('_marker', '')
            if last_marker == '':
                break
            start_time = last_marker

        return indicators

    def get_actors(self, start_time):
        """Get all the actors that were updated after a certain moment in time (UNIX)

        :param start_time: unix time of the oldest actor you want to pull
        """
        actors = []
        offset = 0
        total = 0
        first_run = True

        while offset < total or first_run:
            headers = {
                'Authorization': 'Bearer ' + self.auth_token,
                'Content-Type': 'application/json',
            }
            params = {"sort": "last_modified_date.asc",
                      "filter": 'last_modified_date:>{}'.format(start_time),
                      'limit': self.request_size_limit,
                      'offset': offset}

            resp_json = self.__send_request_check_status(self.actors_path, headers, params)

            total = resp_json.get('meta', {}).get('pagination', {}).get('total')
            offset += resp_json.get('meta', {}).get('pagination', {}).get('limit')
            first_run = False

            actors.extend(resp_json.get('resources', []))

        return actors

    def __check_metadata(self, resp_json):
        if (resp_json.get('meta', {}).get('pagination', {}).get('total') is None) \
                or (resp_json.get('meta', {}).get('pagination', {}).get('limit') is None):
            raise Exception('Unable to decode pagination metadata from response. Response is {}.'.format(resp_json))

    def __send_request_check_status(self, path, headers, params):
        resp = requests.get(path, params=params, headers=headers, verify=False, allow_redirects=True)
        if resp.status_code == http.HTTPStatus.FORBIDDEN:
            logging.warning("Acess denied when trying to get objects from Crowdstrike Intel API. "
                            "Trying to refresh the authentification token.")
            self.__get_auth_token()
            headers['Authorization'] = 'Bearer ' + self.auth_token
            resp = requests.get(path, params=params, headers=headers, verify=False,
                                allow_redirects=True)

        if resp.status_code != http.HTTPStatus.OK:
            raise Exception("Unable to get objects from Crowdstrike Intel API. "
                            "Status code was {}. Response is {}. Request params {}, headers {}.".format(resp.status_code, resp, params, headers))

        resp_json = resp.json()
        self.__check_metadata(resp_json)
        return resp_json


class ReportsImporter:
    """Tool used to import reports from the Crowdstrike Intel API and push them as events in MISP through the MISP API.

    :param misp_client: client for a MISP instance
    :param intel_api_client: client for the Crowdstrike Intel API
    """
    def __init__(self, misp_client, intel_api_client, crowdstrike_org_uuid, reports_timestamp_filename):
        self.misp = misp_client
        self.intel_api_client = intel_api_client
        self.reports_timestamp_filename = reports_timestamp_filename

        org = MISPOrganisation()
        org.uuid = crowdstrike_org_uuid
        self.crowdstrike_org = self.misp.get_organisation(org, True)

    def process_reports(self, reports_days_before, events_already_imported):
        """Pull and process reports.

        :param reports_days_before: in case on an initialisation run, this is the age of the reports pulled in days
        :param events_already_imported: the events already imported in misp, to avoid duplicates
        """
        start_get_events = int((datetime.date.today() - datetime.timedelta(reports_days_before)).strftime("%s"))
        if os.path.isfile(self.reports_timestamp_filename):
            with open(self.reports_timestamp_filename, 'r') as timestampFile:
                line = timestampFile.readline()
                start_get_events = int(line)

        logging.info("Started getting reports from Crowdstrike Intel API and pushing them as events in MISP.")
        time_send_request = datetime.datetime.now()
        reports = self.intel_api_client.get_reports(start_get_events)
        logging.info("Got {} reports from the Crowdstrike Intel API.".format(len(reports)))

        if len(reports) == 0:
            with open(self.reports_timestamp_filename, 'w') as timestampFile:
                timestampFile.write(time_send_request.strftime("%s"))
        else:
            for report in reports:
                report_name = report.get('name')
                if report_name is not None:
                    if events_already_imported.get(report_name) is not None:
                        continue

                event = self.__create_event_from_report(report)
                if event is not None:
                    try:
                        event = self.misp.add_event(event, True)
                        for tag in settings.reports_tags:
                            self.misp.tag(event, tag)
                        if report_name is not None:
                            events_already_imported[report_name] = True
                    except:
                        logging.warning("Could not add or tag event {}.".format(event.info))
                else:
                    logging.warning("Failed to create a MISP event for report {}.".format(report))

                if report.get('last_modified_date') is None:
                    logging.warning("Failed to confirm report {} in file.".format(report))
                    continue

                with open(self.reports_timestamp_filename, 'w') as timestampFile:
                    timestampFile.write(str(report.get('last_modified_date')))

        logging.info("Finished getting reports from Crowdstrike Intel API and pushing them as events in MISP.")

    def __create_event_from_report(self, report):
        event = MISPEvent()
        event.analysis = 2
        event.orgc = self.crowdstrike_org

        if report.get('name'):
            event.info = report.get('name')
        else:
            logging.warning("Report {} missing name field.".format(report.get('id')))

        if report.get('url'):
            event.add_attribute('link', report.get('url'))
        else:
            logging.warning("Report {} missing url field.".format(report.get('id')))

        if report.get('short_description'):
            event.add_attribute('comment', report.get('short_description'))
        else:
            logging.warning("Report {} missing short_description field.".format(report.get('id')))

        for actor in report.get('actors', []):
            if actor.get('name'):
                event.add_attribute('threat-actor', actor.get('name'))
            else:
                logging.warning("Actor from report {} missing name field.".format(report.get('id')))

        for country in report.get('target_countries', []):
            if country.get('value'):
                country_object = MISPObject('victim')
                country_object.add_attribute('regions', country.get('value'))
                event.add_object(country_object)
            else:
                logging.warning("Target country from report {} missing value field.".format(report.get('id')))

        for industry in report.get('target_industries', []):
            if industry.get('value'):
                industry_object = MISPObject('victim')
                industry_object.add_attribute('sectors', industry.get('value'))
                event.add_object(industry_object)
            else:
                logging.warning("Target industry from report {} missing value field.".format(report.get('id')))

        return event


class IndicatorsImporter:
    """Tool used to import indicators from the Crowdstrike Intel API and add them as objects attached to the events in
    MISP coresponding to the Crowdstrike Intel Reports they are related to.

    :param misp_client: client for a MISP instance
    :param intel_api_client: client for the Crowdstrike Intel API
    """
    def __init__(self, misp_client, intel_api_client, crowdstrike_org_uuid, indicators_timestamp_filename, import_all_indicators, delete_outdated):
        self.misp = misp_client
        self.intel_api_client = intel_api_client
        self.indicators_timestamp_filename = indicators_timestamp_filename
        self.import_all_indicators = import_all_indicators
        self.delete_outdated = delete_outdated

        org = MISPOrganisation()
        org.uuid = crowdstrike_org_uuid
        self.crowdstrike_org = self.misp.get_organisation(org, True)



        self.reportsIDS = {}

    def __get_cs_reports_from_misp(self):
        events = self.misp.search_index(tags=[settings.reports_unique_tag])
        for event in events:
            if event.get('info'):
                self.reportsIDS[event.get('info').split(' ', 1)[0]] = event
            else:
                logging.warning("Event {} missing info field.".format(event))

    def process_indicators(self, indicators_days_before, events_already_imported):
        """Pull and process indicators.

        :param indicators_days_before: in case on an initial run, this is the age of the indicators pulled in days
        :param events_already_imported: the events already imported in misp, to avoid duplicates
        """
        start_get_events = int((datetime.date.today() - datetime.timedelta(indicators_days_before)).strftime("%s"))
        if os.path.isfile(self.indicators_timestamp_filename):
            with open(self.indicators_timestamp_filename, 'r') as timestampFile:
                line = timestampFile.readline()
                start_get_events = int(line)

        logging.info("Started getting indicators from Crowdstrike Intel API and pushing them in MISP.")
        time_send_request = datetime.datetime.now()
        indicators = self.intel_api_client.get_indicators(start_get_events, self.delete_outdated)
        logging.info("Got {} indicators from the Crowdstrike Intel API.".format(len(indicators)))

        if len(indicators) == 0:
            with open(self.indicators_timestamp_filename, 'w') as timestampFile:
                timestampFile.write(time_send_request.strftime("%s"))
        else:
            self.__get_cs_reports_from_misp()
            self.__push_indicators(indicators, events_already_imported)

        logging.info("Finished getting indicators from Crowdstrike Intel API and pushing them in MISP.")

    def __push_indicators(self, indicators, events_already_imported):
        for indicator in indicators:
            if self.import_all_indicators or len(indicator.get('reports', [])) > 0:

                indicator_name = indicator.get('indicator')

                if self.delete_outdated and indicator_name is not None and indicator.get('deleted', False):
                    events = self.misp.search_index(eventinfo=indicator_name, pythonify=True)
                    for event in events:
                        self.misp.delete_event(event)
                        try:
                            events_already_imported.pop(indicator_name)
                        except:
                            logging.debug('indicator {} was marked as deleted in intel API but is not stored in MISP. skipping'.format(indicator_name))
                        logging.warning('deleted indicator {}'.format(indicator_name))
                    continue

                if indicator_name is not None:
                    if events_already_imported.get(indicator_name) is not None:
                        continue

                self.__create_object_for_indicator(indicator)

                related_to_a_misp_report = False
                indicator_value = indicator.get('indicator')
                if indicator_value:
                    for report in indicator.get('reports', []):
                        event = self.reportsIDS.get(report)
                        if event:
                            related_to_a_misp_report = True
                            indicator_object = self.__create_object_for_indicator(indicator)
                            if indicator_object:
                                try:
                                    if isinstance(indicator_object, MISPObject):
                                        self.misp.add_object(event, indicator_object, True)
                                    elif isinstance(indicator_object, MISPAttribute):
                                        self.misp.add_attribute(event, indicator_object, True)
                                except:
                                    logging.warning("Could not add object or attribute {} for "
                                                    "event {}.".format(indicator_object, event))
                else:
                    logging.warning("Indicator {} missing indicator field.".format(indicator.get('id')))

                if related_to_a_misp_report or self.import_all_indicators:
                    self.__add_indicator_event(indicator)
                    if indicator_name is not None:
                        events_already_imported[indicator_name] = True

            if indicator.get('last_updated') is None:
                logging.warning("Failed to confirm indicator {} in file.".format(indicator))
                continue

            with open(self.indicators_timestamp_filename, 'w') as timestampFile:
                timestampFile.write(str(indicator.get('last_updated')))

    def __add_indicator_event(self, indicator):
        event = MISPEvent()
        event.analysis = 2
        event.orgc = self.crowdstrike_org

        indicator_value = indicator.get('indicator')
        if indicator_value:
            event.info = indicator_value
            indicator_object = self.__create_object_for_indicator(indicator)
            if indicator_object:
                if isinstance(indicator_object, MISPObject):
                    event.add_object(indicator_object)
                elif isinstance(indicator_object, MISPAttribute):
                    event.add_attribute(indicator_object.type, indicator_object.value)
                else:
                    logging.warning("Couldn't add indicator object to the event corresponding to MISP "
                                    "event {}.".format(indicator_value))
        else:
            logging.warning("Indicator {} missing indicator field.".format(indicator.get('id')))

        malicious_confidence = indicator.get('malicious_confidence')
        if malicious_confidence is None:
            logging.warning("Indicator {} missing malicious_confidence field.".format(indicator.get('id')))
        elif malicious_confidence == 'unverified':
            event.threat_level_id = 4
        elif malicious_confidence == 'low':
            event.threat_level_id = 3
        elif malicious_confidence == 'medium':
            event.threat_level_id = 2
        elif malicious_confidence == 'high':
            event.threat_level_id = 1
        else:
            logging.warning("Could not map malicious_confidence level with value {}".format(malicious_confidence))

        for actor in indicator.get('actors', []):
            event.add_attribute('threat-actor', actor)

        for target in indicator.get('targets', []):
            industry_object = MISPObject('victim')
            industry_object.add_attribute('sectors', target)
            event.add_object(industry_object)

        try:
            event = self.misp.add_event(event, True)
            for tag in settings.indicators_tags:
                self.misp.tag(event, tag)
        except:
            logging.warning("Could not add or tag event {}.".format(event.info))

        for malware_family in indicator.get('malware_families', []):
            galaxy = settings.malware_family_to_galaxy_mapping.get(malware_family)
            if galaxy is not None:
                try:
                    self.misp.tag(event, galaxy)
                except:
                    logging.warning("Could not add event {} in galaxy/cluster.".format(event.info))
            else:
                logging.info("Don't know how to map malware_family {} to a MISP galaxy.".format(malware_family))

    def __create_object_for_indicator(self, indicator):
        if not indicator.get('type') or not indicator.get('indicator'):
            logging.warning("Indicator {} missing type or indicator field.".format(indicator.get('id')))
            return

        indicator_type = indicator.get('type')
        indicator_value = indicator.get('indicator')

        if indicator_type == 'hash_md5':
            indicator_object = MISPObject('file')
            indicator_object.add_attribute('md5', indicator_value)
            return indicator_object
        elif indicator_type == 'hash_sha256':
            indicator_object = MISPObject('file')
            indicator_object.add_attribute('sha256', indicator_value)
            return indicator_object
        elif indicator_type == 'hash_sha1':
            indicator_object = MISPObject('file')
            indicator_object.add_attribute('sha1', indicator_value)
            return indicator_object
        elif indicator_type == 'file_name':
            indicator_object = MISPObject('file')
            indicator_object.add_attribute('filename', indicator_value)
            return indicator_object
        elif indicator_type == 'domain':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Network activity'
            indicator_attribute.type = 'domain'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'mutex_name':
            indicator_object = MISPObject('mutex')
            indicator_object.add_attribute('name', indicator_value)
            return indicator_object
        elif indicator_type == 'password':
            indicator_object = MISPObject('credential')
            indicator_object.add_attribute('password', indicator_value)
            return indicator_object
        elif indicator_type == 'url':
            indicator_object = MISPObject('url')
            indicator_object.add_attribute('url', indicator_value)
            return indicator_object
        elif indicator_type == 'campaign_id':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Attribution'
            indicator_attribute.type = 'campaign-id'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'ip_address':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Network activity'
            indicator_attribute.type = 'ip-src'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'email_address':
            indicator_object = MISPObject('email')
            indicator_object.add_attribute('reply-to', indicator_value)
            return indicator_object
        elif indicator_type == 'username':
            indicator_object = MISPObject('credential')
            indicator_object.add_attribute('username', indicator_value)
            return indicator_object
        elif indicator_type == 'bitcoin_address':
            indicator_object = MISPObject('btc-transaction')
            indicator_object.add_attribute('btc-address', indicator_value)
            return indicator_object
        elif indicator_type == 'registry':
            indicator_object = MISPObject('registry-key')
            indicator_object.add_attribute('key', indicator_value)
            return indicator_object
        elif indicator_type == 'x509_serial':
            indicator_object = MISPObject('x509')
            indicator_object.add_attribute('serial-number', indicator_value)
            return indicator_object
        elif indicator_type == 'file_path':
            indicator_object = MISPObject('file')
            indicator_object.add_attribute('fullpath', indicator_value)
            return indicator_object
        elif indicator_type == 'email_subject':
            indicator_object = MISPObject('email')
            indicator_object.add_attribute('subject', indicator_value)
            return indicator_object
        elif indicator_type == 'service_name':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Artifacts Dropped'
            indicator_attribute.type = 'windows-service-name'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'user_agent':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Network activity'
            indicator_attribute.type = 'user-agent'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'coin_address':
            indicator_object = MISPObject('coin-address')
            indicator_object.add_attribute('address', indicator_value)
            return indicator_object
        elif indicator_type == 'x509_subject':
            indicator_object = MISPObject('x509')
            indicator_object.add_attribute('subject', indicator_value)
            return indicator_object
        elif indicator_type == 'port':
            indicator_attribute = MISPAttribute()
            indicator_attribute.category = 'Network activity'
            indicator_attribute.type = 'port'
            indicator_attribute.value = indicator_value
            return indicator_attribute
        elif indicator_type == 'device_name':
            indicator_object = MISPObject('device')
            indicator_object.add_attribute('name', indicator_value)
            return indicator_object
        elif indicator_type == 'hash_imphash':
            indicator_object = MISPObject('pe')
            indicator_object.add_attribute('imphash', indicator_value)
            return indicator_object
        else:
            logging.warning("Unable to map indicator type {} to a MISP object or "
                            "attribute.".format(indicator.get('type')))


class ActorsImporter:
    """Tool used to import actors from the Crowdstrike Intel API and push them as events in MISP through the MISP API.

    :param misp_client: client for a MISP instance
    :param intel_api_client: client for the Crowdstrike Intel API
    """
    def __init__(self, misp_client, intel_api_client, crowdstrike_org_uuid, actors_timestamp_filename):
        self.misp = misp_client
        self.intel_api_client = intel_api_client
        self.actors_timestamp_filename = actors_timestamp_filename
        org = MISPOrganisation()
        org.uuid = crowdstrike_org_uuid
        self.crowdstrike_org = self.misp.get_organisation(org, True)

    def process_actors(self, actors_days_before, events_already_imported):
        """Pull and process actors.

        :param actors_days_before: in case on an initialisation run, this is the age of the actors pulled in days
        :param events_already_imported: the events already imported in misp, to avoid duplicates
        """
        start_get_events = int((datetime.date.today() - datetime.timedelta(actors_days_before)).strftime("%s"))
        if os.path.isfile(self.actors_timestamp_filename):
            with open(self.actors_timestamp_filename, 'r') as timestampFile:
                line = timestampFile.readline()
                start_get_events = int(line)

        logging.info("Started getting actors from Crowdstrike Intel API and pushing them as events in MISP.")
        time_send_request = datetime.datetime.now()
        actors = self.intel_api_client.get_actors(start_get_events)
        logging.info("Got {} actors from the Crowdstrike Intel API.".format(len(actors)))

        if len(actors) == 0:
            with open(self.actors_timestamp_filename, 'w') as timestampFile:
                timestampFile.write(time_send_request.strftime("%s"))
        else:
            for actor in actors:

                actor_name = actor.get('name')
                if actor_name is not None:
                    if events_already_imported.get(actor_name) is not None:
                        continue

                event = self.__create_event_from_actor(actor)
                if not event:
                    logging.warning("Failed to create a MISP event for actor {}.".format(actor))
                    continue

                try:
                    event = self.misp.add_event(event, True)
                    for tag in settings.actors_tags:
                        self.misp.tag(event, tag)
                    if actor_name is not None:
                        events_already_imported[actor_name] = True
                except:
                    logging.warning("Could not add or tag event {}.".format(event.info))

                if actor.get('last_modified_date') is None:
                    logging.warning("Failed to confirm actor {} in file.".format(actor))
                    continue

                with open(self.actors_timestamp_filename, 'w') as timestampFile:
                    timestampFile.write(str(actor.get('last_modified_date')))

        logging.info("Finished getting actors from Crowdstrike Intel API and pushing them as events in MISP.")

    def __create_event_from_actor(self, actor):
        event = MISPEvent()
        event.analysis = 2
        event.orgc = self.crowdstrike_org

        if actor.get('name'):
            event.info = actor.get('name')
        else:
            logging.warning("Actor {} missing field name.".format(actor.get('id')))

        if actor.get('url'):
            event.add_attribute('link', actor.get('url'))
        else:
            logging.warning("Actor {} missing field url.".format(actor.get('id')))

        if actor.get('short_description'):
            event.add_attribute('comment', actor.get('short_description'))
        else:
            logging.warning("Actor {} missing field short_description.".format(actor.get('id')))

        if actor.get('known_as'):
            known_as_object = MISPObject('organization')
            known_as_object.add_attribute('alias', actor.get('known_as'))
            event.add_object(known_as_object)
        else:
            logging.warning("Actor {} missing field known_as.".format(actor.get('id')))

        had_timestamp = False
        timestamp_object = MISPObject('timestamp')

        if actor.get('first_activity_date'):
            timestamp_object.add_attribute('first-seen', datetime.datetime.utcfromtimestamp(actor.get('first_activity_date')).isoformat())
            had_timestamp = True
        else:
            logging.warning("Actor {} missing field first_activity_date.".format(actor.get('id')))

        if actor.get('last_activity_date'):
            timestamp_object.add_attribute('last-seen', datetime.datetime.utcfromtimestamp(actor.get('last_activity_date')).isoformat())
            had_timestamp = True
        else:
            logging.warning("Actor {} missing field last_activity_date.".format(actor.get('id')))

        if had_timestamp:
            event.add_object(timestamp_object)

        for country in actor.get('target_countries', []):
            if country.get('value'):
                country_object = MISPObject('victim')
                country_object.add_attribute('regions', country.get('value'))
                event.add_object(country_object)
            else:
                logging.warning("Target country from actor {} is missing value field.".format(actor.get('id')))

        for industry in actor.get('target_industries', []):
            if industry.get('value'):
                industry_object = MISPObject('victim')
                industry_object.add_attribute('sectors', industry.get('value'))
                event.add_object(industry_object)
            else:
                logging.warning("Target country from actor {} is missing value field.".format(actor.get('id')))

        return event


class CrowdstrikeToMISPImporter:
    """Tool used to import indicators and reports from the Crowdstrike Intel API.

    :param intel_api_client: client for the Crowdstrike Intel API
    """
    def __init__(self, intel_api_client, misp_url, misp_auth_key, crowdstrike_org_uuid, reports_timestamp_filename,
                 indicators_timestamp_filename, actors_timestamp_filename, reports, relevant_indicators, all_indicators, delete_outdated_indicators, actors):
        self.intelAPIClient = intel_api_client
        self.mispClient = ExpandedPyMISP(misp_url, misp_auth_key, False, False)
        self.reports = reports
        self.relevant_indicators = relevant_indicators
        self.all_indicators = all_indicators
        self.actors = actors
        self.delete_outdated_indicators = delete_outdated_indicators

        self.eventIDS = {}

        if self.reports:
            self.reportsImporter = ReportsImporter(self.mispClient, self.intelAPIClient, crowdstrike_org_uuid,
                                                   reports_timestamp_filename)
        if self.relevant_indicators or self.all_indicators:
            self.indicatorsImporter = IndicatorsImporter(self.mispClient, self.intelAPIClient, crowdstrike_org_uuid,
                                                         indicators_timestamp_filename, self.all_indicators, self.delete_outdated_indicators)
        if self.actors:
            self.actorsImporter = ActorsImporter(self.mispClient, self.intelAPIClient, crowdstrike_org_uuid,
                                                 actors_timestamp_filename)



    def clean_crowdstrike_events(self, clean_reports, clean_indicators, clean_actors):
        """Delete events from a MISP instance.
        """
        tags = []
        if clean_reports:
            tags.append(settings.reports_unique_tag)
        if clean_indicators:
            tags.append(settings.indicators_unique_tag)
        if clean_actors:
            tags.append(settings.actors_unique_tag)

        if clean_reports or clean_indicators or clean_actors:
            events = self.mispClient.search_index(tags=tags)
            for event in events:
                self.mispClient.delete_event(event)
            logging.info("Finished cleaning up Crowdstrike related events from MISP.")

    def clean_old_crowdstrike_events(self, max_age):
        if max_age is not None:
            timestamp_max = int((datetime.date.today() - datetime.timedelta(max_age)).strftime("%s"))
            events = self.mispClient.search(tags=[settings.reports_unique_tag, settings.indicators_unique_tag, settings.actors_unique_tag], timestamp=[0, timestamp_max])
            for event in events:
                self.mispClient.delete_event(event)
            logging.info("Finished cleaning up Crowdstrike related events from MISP.")

    def import_from_crowdstrike(self, reports_days_before, indicators_days_before, actors_days_before):
        """Import reports and events from Crowdstrike Intel API.

        :param reports_days_before: in case on an initial run, this is the age of the reports pulled in days
        :param indicators_days_before: in case on an initial run, this is the age of the indicators pulled in days
        :param actors_days_before: in case on an initial run, this is the age of the actors pulled in days
        """
        if self.reports:
            self.reportsImporter.process_reports(reports_days_before, self.eventIDS)
        if self.relevant_indicators or self.all_indicators:
            self.indicatorsImporter.process_indicators(indicators_days_before, self.eventIDS)
        if self.actors:
            self.actorsImporter.process_actors(actors_days_before, self.eventIDS)

    def import_from_misp(self, tags):
        events = self.mispClient.search_index(tags=tags)
        for event in events:
            if event.get('info'):
                self.eventIDS[event.get('info')] = True
            else:
                logging.warning("Event {} missing info field.".format(event))


def main():
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    parser = argparse.ArgumentParser(description="Tool used to import reports and indicators from Crowdstrike Intel"
                                                 "API into a MISP instance.")
    parser.add_argument("--clean_reports", action="store_true", help="Set this to run a cleaning round on reports.")
    parser.add_argument("--clean_indicators", action="store_true", help="Set this to run a cleaning round on indicators.")
    parser.add_argument("--clean_actors", action="store_true", help="Set this to run a cleaning round on actors,")
    parser.add_argument("--debug", action="store_true", help="Set this to activate debug logs.")
    parser.add_argument("--max_age", type=int, help="Maximum age of the objects to be stored in MISP in days. Objects older than that will be deleted.")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("--related_indicators", action="store_true", help="Set this to only import indicators related to reports.")
    group.add_argument("--all_indicators", action="store_true", help="Set this to import all indicators.")
    parser.add_argument("--delete_outdated_indicators", action='store_true', help="Set this to check if the indicators you are imported have been marked as deleted and if they"
                                                                                  "have been already inserted, delete them.")
    parser.add_argument("--reports", action="store_true", help="Set this to import reports.")
    parser.add_argument("--actors", action="store_true", help="Set this to import actors.")
    args = parser.parse_args()

    logging.root.setLevel(logging.INFO)
    if args.debug:
        logging.root.setLevel(logging.DEBUG)

    if args.clean_reports or args.clean_indicators or args.clean_actors:
        intel_api_client = IntelAPIClient(settings.client_id, settings.client_secret, settings.crowdstrike_url)
        importer = CrowdstrikeToMISPImporter(intel_api_client, settings.misp_url, settings.misp_auth_key,
                                             settings.crowdstrike_org_uuid, settings.reports_timestamp_filename,
                                             settings.indicators_timestamp_filename, settings.actors_timestamp_filename,
                                             args.reports, args.related_indicators, args.all_indicators, args.delete_outdated_indicators, args.actors)
        try:
            importer.clean_crowdstrike_events(args.clean_reports, args.clean_indicators, args.clean_actors)
            if args.clean_reports and os.path.isfile(settings.reports_timestamp_filename):
                os.remove(settings.reports_timestamp_filename)
            if args.clean_indicators and os.path.isfile(settings.indicators_timestamp_filename):
                os.remove(settings.indicators_timestamp_filename)
            if args.clean_actors and os.path.isfile(settings.actors_timestamp_filename):
                os.remove(settings.actors_timestamp_filename)
        except Exception as e:
            logging.exception(e)
            sys.exit(1)

    if args.reports or args.actors or args.related_indicators or args.all_indicators:
        intel_api_client = IntelAPIClient(settings.client_id, settings.client_secret, settings.crowdstrike_url)
        importer = CrowdstrikeToMISPImporter(intel_api_client, settings.misp_url, settings.misp_auth_key,
                                             settings.crowdstrike_org_uuid, settings.reports_timestamp_filename,
                                             settings.indicators_timestamp_filename, settings.actors_timestamp_filename,
                                             args.reports, args.related_indicators, args.all_indicators, args.delete_outdated_indicators, args.actors)
        try:
            tags = []

            if args.reports:
                tags.append(settings.reports_unique_tag)

            if args.related_indicators or args.all_indicators:
                tags.append(settings.indicators_unique_tag)

            if args.actors:
                tags.append(settings.actors_unique_tag)

            importer.import_from_misp(tags)
            importer.import_from_crowdstrike(settings.init_reports_days_before, settings.init_indicators_days_before,
                                             settings.init_actors_days_before)
        except Exception as e:
            logging.exception(e)
            sys.exit(1)

    if args.max_age is not None:
        intel_api_client = IntelAPIClient(settings.client_id, settings.client_secret, settings.crowdstrike_url)
        importer = CrowdstrikeToMISPImporter(intel_api_client, settings.misp_url, settings.misp_auth_key,
                                             settings.crowdstrike_org_uuid, settings.reports_timestamp_filename,
                                             settings.indicators_timestamp_filename, settings.actors_timestamp_filename,
                                             args.reports, args.related_indicators, args.all_indicators, args.delete_outdated_indicators, args.actors)
        try:
            importer.clean_old_crowdstrike_events(args.max_age)
        except Exception as e:
            logging.exception(e)
            sys.exit(1)
            

if __name__ == '__main__':
    main()
